class CircleNote():
    '''
    CircleNote is for displaying notes, universal
    '''
    def __init__(self, **kwargs):
        
        #more for drawing part
        self.noteFaceColor = kwargs.get('noteFaceColor', 'blue')
        self.noteEdgeColor = kwargs.get('noteEdgeColor', 'black')
        self.noteLineWidth = kwargs.get('noteLineWidth', 2)
        self.noteFill = kwargs.get('noteFill', True)
        self.noteRadius = kwargs.get('noteRadius', 0.4)
        
        #text part
        self.intervals = kwargs.get('interval', None)
        self.scaleDegrees = kwargs.get('scaleDegree', None)
        self.name = kwargs.get('name', None)
        self.nameWithOctave = kwargs.get('nameWithOctave', None)

        #current note string and fret
        self.fret = kwargs.get('fret', None)
        self.string = kwargs.get('string', None)
    
    def getNoteRadius(self):
        return self.noteRadius
    
    def setNoteRadius(self,radius):
        self.noteRadius = radius
        
    def getNoteFaceColor(self):
        return self.noteFaceColor
    
    def setNoteFaceColor(self,color):
        self.noteFaceColor = color
        
    def getNoteEdgeColor(self):
        return self.noteEdgeColor
    
    def setNoteEdgeColor(self,color):
        self.noteEdgeColor = color
        
    def getNoteLineWidth(self):
        return self.noteLineWidth
    
    def setNoteLineWidth(self,lw):
        self.noteLineWidth = lw
        
    def getNoteFill(self):
        return self.noteFill
    
    def setNoteFill(self,noteFill):
        self.noteFill = noteFill
    
    def getIntervals(self):
        return self.intervals
    
    def setIntervals(self, interval):
        self.intervals = interval
    
    def getScaleDegrees(self):
        return self.scaleDegrees
    
    def setScaleDegrees(self, scaleDegree):
        self.scaleDegrees = scaleDegree
    
    def getName(self):
        return self.name
    
    def setName(self, name):
        self.name = name
    
    def getNameWithOctave(self):
        return self.nameWithOctave
    
    def setNameWithOctave(self, nameWithOctave):
        self.nameWithOctave = nameWithOctave

    


from bokeh.plotting import figure
from bokeh.models import Range1d

class FretboardFigure():
    def __init__(self, note, theme, orientation="h", width = None, height=None):
        self.fig = figure()
        self.figHorXRange = Range1d(-8*note.getNoteRadius(),
                                        (theme.fretboardRange.numOfFrets+1.3)*theme.fretboardDesign.distanceBetweenFrets)
        self.figHorYRange = Range1d(-3*note.getNoteRadius(),
                                        theme.fretboardDesign.distanceBetweenStrings*theme.tuning.numOfStrings)
        self.figVerXRange = Range1d(-3*note.getNoteRadius(),
                                        theme.fretboardDesign.distanceBetweenStrings*theme.tuning.numOfStrings)
        self.figVerYRange = Range1d(-8*note.getNoteRadius(),
                                        (theme.fretboardRange.numOfFrets+2)*theme.fretboardDesign.distanceBetweenFrets)

        if (orientation == "h"):
            if(width == None or height == None):
                self.fig.width = 800
                self.fig.height = 400
            else:
                self.fig.width = width
                self.fig.height = height
            self.fig.x_range = self.figHorXRange
            self.fig.y_range = self.figHorYRange
        
        else:
            if(width == None or height == None):
                self.fig.width = 400
                self.fig.height = 800
            else:
                self.fig.width = width
                self.fig.height = height
            self.fig.x_range = self.figVerXRange
            self.fig.y_range = self.figVerYRange

        self.stringLabel = ""
        self.fretLabel = ""

    @property
    def stringLabel(self):
        return self._stringLabel

    @stringLabel.setter
    def stringLabel(self, stringLabel):
        self._stringLabel = stringLabel

    def addStringLabelLayout(self):
        self.fig.add_layout(self.stringLabel)

    @property
    def fretLabel(self):
        return self._fretLabel

    @fretLabel.setter
    def fretLabel(self, fretLabel):
        self._fretLabel = fretLabel
    
    def addFretLabelLayout(self):
        self.fig.add_layout(self.fretLabel)

    @property
    def fig(self):
        return self._fig

    @fig.setter
    def fig(self, fig):
        self._fig = fig

    @property
    def width(self):
        return self.fig.width

    @width.setter
    def width(self, width):
        self.fig.width = width

    @property
    def height(self):
        return self.fig.height
    
    @height.setter
    def height(self, height):
        self.fig.height = height

    def getFigHorXRange(self):
        return self.figHorXRange

    def setFigHorXRange(self, v1, v2):
        self.figHorXRange = Range1d(v1, v2)

    def getFigHorYRange(self):
        return self.figHorYRange

    def setFigHorYRange(self, v1, v2):
        self.figHorYRange = Range1d(v1, v2)

    def getFigVerXRange(self):
        return self.figVerXRange

    def setFigVerXRange(self, v1, v2):
        self.figVerXRange = Range1d(v1, v2)

    def getFigVerYRange(self):
        return self.figVerYRange

    def setFigVerYRange(self, v1, v2):
        self.figVerYRange = Range1d(v1, v2)
        


class VideoNote():
    def __init__(self, pitch, startTime, endTime):
        self.pitch = pitch
        self.startTime = startTime
        self.endTime = endTime
    
    def getPitch(self):
        return self.pitch
    
    def setPitch(self, pitch):
        self.pitch = pitch
    
    def getStartTime(self):
        return self.startTime
    
    def setStartTime(self, startTime):
        self.startTime = startTime

    def getEndTime(self):
        return self.endTime
    
    def setEndTime(self, endTime):
        self.endTime = endTime



import os
from Utilities.Constants import BASE_PATH

class PathInfo():
    def __init__(self, name, path):
        self.path = path
        self.name = name
        self.pathWithName = os.path.join(self.path, self.name)

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, value):
        self._path = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    def getPathWithName(self):
        return os.path.join(self._path, self._name)

    def setPathWithName(self, value):
        # Split the provided value to separate path and name
        directory, filename = os.path.split(value)
        self._path = directory
        self._name = filename

class EmbedPathInfo(PathInfo):
    def __init__(self, name="default.html"):
     self.path = os.path.join(BASE_PATH, 'Outputs', 'Embeds')
     self.name = name
     self.pathWithName = os.path.join(self.path, self.name)


import os
from tqdm import tqdm
from PIL import Image
import re
import ffmpeg
import cv2

from bokeh.io import export_png, export_svg
import glob

class VideoManager():
    def __init__(self, fretboard, video, images, imageProgressBar=True):
        self.fretboard = fretboard
        self.video = video
        self.images = images
        self.imageProgressBar = imageProgressBar

    def saveAsVideoImages(self):
        oriImgName = self.images.name
        print(oriImgName)
        for k, v in self.video.getFramesItems():
            self.fretboard.updateFretboard(v)
            self.images.name = str(k)+oriImgName
            self.saveImage()
            print("saving"+self.images.name)
        print("done")

    # for guitarset n other data where num of second is not defined
    def saveAsVideoImagesNoSeconds(self):
        oriImgName = self.images.name
        images = {}
        print("IMAGES Generateing")
        for i in tqdm(range(len(self.video.getFrames())), disable=not(self.imageProgressBar)):
            frame = self.video.getFrames()[i]
            self.images.name = str(i)+oriImgName

            if frame in images:
                image = images[frame]
                image.copy().save(os.path.join(
                    self.images.outputPathName, self.images.name + self.images.meta))
            else:
                self.fretboard.updateFretboard(self.video.getFrames()[i])
                self.saveImage()
                image = Image.open(os.path.join(
                    self.images.outputPathName, self.images.name + self.images.meta))
                images[frame] = image.copy()
        print("IMAGES Generate done")

    def saveAsVideo(self):
        images = os.listdir(self.images.outputPathName)
        images = sorted(images, key=lambda s: [
                        int(x) if x.isdigit() else x for x in re.split('(\d+)', s)])

        fourcc = cv2.VideoWriter_fourcc(*self.video.getCodec())
        frameSize = (self.fretboard.fretboardFig.fig.width, self.fretboard.fretboardFig.fig.height)

        videoWriter = cv2.VideoWriter(self.video.getVideoPathWithName(
        )+self.video.getFileExtension(), fourcc, self.video.getFrameRate(), frameSize)

        for image in images:
            frame = cv2.imread(os.path.join(self.images.outputPathName, image))
            videoWriter.write(frame)

        cv2.destroyAllWindows()
        videoWriter.release()

        print("VIDEO " + self.video.getVideoName() +
              " saved at "+self.video.getVideoPathName())

    def createVideoWithAudio(self):
        self.saveAsVideo()

        videoPath = ffmpeg.input(
            self.video.getVideoPathWithName()+self.video.getFileExtension())
        audioPath = ffmpeg.input(self.video.getAudioPathWithName())


        ffmpeg.concat(videoPath, audioPath, v=1, a=1).output(
            self.video.getVideoWAudioPathWithName()).run(overwrite_output=True)
        print("video save with audio done")

    def saveVideoWithAudio(self):
        videoPath = ffmpeg.input(
            self.video.getVideoPathWithName()+self.video.getFileExtension())
        audioPath = ffmpeg.input(self.video.getAudioPathWithName())

        ffmpeg.concat(videoPath, audioPath, v=1, a=1).output(
            self.video.getVideoName(
            )+".mp4").run(overwrite_output=True)
        print("video save with audio done")
    
    def saveImage(self):
        if (self.images.meta.lower() == ".png"):
            export_png(self.fretboard.getFretboardFig().fig, filename=self.images.fileName)

        elif (self.images.meta.lower() == ".svg"):
            export_svg(self.fretboard.getFretboardFig().fig, filename=self.images.fileName)


    def deleteAllImages(self):
        files = glob.glob(os.path.join(self.images.outputPathName, "*"))
        for f in files:
            os.remove(f)
        print("All Images Delete")

        
    @property
    def imageProgressBar(self):
        return self._imageProgressBar

    @imageProgressBar.setter
    def imageProgressBar(self, imageProgressBar):
        self._imageProgressBar = imageProgressBar


from setuptools import setuptools, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='seeFretboard',
    version='0.0.1.1',
    author='Linda Rong Zhang',
    author_email='ronglindaz@gmail.com',
    description='Testing installation of Package',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/LindaRZhang/seeFretboard',
    project_urls = {
        "Bug Tracker": "https://github.com/LindaRZhang/seeFretboard/issues"
    },
    license='MIT',
    packages=find_packages(include=['seeFretboard', 'seeFretboard.*']),
    install_requires=['requests'],
    include_package_data=True
)


import music21 

def intervalsToScaleDegrees(intervals):
    """Converts a list of intervals to scale degrees.
    
    Args:
        intervals (list): A list of interval names (e.g. "M3", "P5")
        
    Returns: 
        scaleDegrees (list): The list of scale degrees (e.g. "3", "5") corresponding to the intervals.
    """
    scaleDegrees = []
    for i in intervals:
        scaleDegrees.append(intervalToScaleDegree(i))
    return scaleDegrees

def scaleDegreesToIntervals(degrees):
    """Converts a list of scale degrees to intervals.
    
    Args:
        degrees (list): A list of scale degree numbers (e.g. "3", "5") 
        
    Returns:
        intervals (list): The list of interval names (e.g. "M3", "P5") corresponding to the scale degrees.
    """
    intervals = []
    for i in degrees:
        intervals.append(scaleDegreeToInterval(i))
    return intervals

def intervalToScaleDegree(interval):
    """Converts an interval name to a scale degree.
    
    Args:
        interval (str): The interval name (e.g. "M3", "P5")
        
    Returns:
        degree (str): The scale degree (e.g. "3", "5") corresponding to the interval.
    """
    index = allInterval.index(interval)
    scaleDegree = scaleDegrees[index % 12] 
    
    return scaleDegree

def scaleDegreeToInterval(degree):
    """Converts a scale degree to an interval name.
    
    Args:
        degree (str): The scale degree (e.g. "3", "5")
        
    Returns:
        interval (str): The interval name (e.g. "M3", "P5") corresponding to the scale degree.
    """
    return allInterval[scaleDegrees.index(degree)]

def midiToFret(string, midiNotes):
        """Converts a MIDI note number to a fret number for a given string.
    
        Args:
            string (int): The string number, where 1 is the highest pitch string.
            midiNotes (list): The list of MIDI note numbers for each string, from lowest to highest pitch.
            
        Returns:
            fret (int): The fret number corresponding to the MIDI note on the given string.
        """
        notes = []
        for m in midiNotes:
            fret = round(abs(int(string) - int(m)))
            notes.append(fret)

        return notes

def fretsToMidi(string, frets):
    """Converts a list of frets on a string to MIDI note numbers.
    
    Args:
        string (int): The string number, where 1 is the highest pitch string.
        frets (list): The list of fret numbers on the string.
        
    Returns:
        midiNotes (list): The list of MIDI note numbers corresponding to the frets.
    """
    midiNotes = []
    for fret in frets:
        midi = fretToMidi(string,fret)
        midiNotes.append(midi)

    return midiNotes

def fretToMidi(string, fret):
    """Converts a fret number on a string to a MIDI note number.
    
    Args:
        string (int): The string number, where 1 is the highest pitch string.
        fret (int): The fret number on the string. 
        
    Returns:
        midi (int): The MIDI note number corresponding to the fret on the string.
    """
    midi = int(string) + int(fret)
    
    return midi

def midiToNoteNameWithOctave(midiNote):
    """Converts a MIDI note number to a note name with octave.
    
    Args:
        midiNote (int): The MIDI note number.
        
    Returns:
        noteNameWithOctave (str): The note name and octave (e.g. "C4", "F#5") corresponding to the MIDI note number.
    """
    note = music21.note.Note()
    note.pitch.midi = midiNote
    return note.nameWithOctave
